-----------------------------------------------------------------------------------------------
-- Client Lua Script for WakeUp
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- WakeUp Module Definition
-----------------------------------------------------------------------------------------------
local WakeUp = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999

local tSounds = {
	{name="Alarm Clock",	file="AlarmClock",	duration=0.45},
	{name="Sword Fight",	file="Blades",		duration=5},
	{name="Laser Beam",		file="DeathRay",	duration=1},
	{name="Crazy Laugh",	file="Hahaha",		duration=12},
	{name="Moo",			file="Moo",			duration=3},
	{name="News Report",	file="News",		duration=5.5},
	{name="Rooster",		file="Rooster",		duration=2},
	{name="Toilet",			file="Toilet",		duration=6.5},
	{name="Turkey Honk",	file="Turkey1",		duration=3},
	{name="Turkey Gobble",	file="Turkey2",		duration=1.1},
}

local tNavi = {
	"NHello",
	"NHey",
	"NLook",
	"NListen",
	"NWatchOut"
}

local tTriggers = {
	["Group Invite"]={
		{"Group_Invited","OnGroupInvite"},
		{"Group_AcceptInvite","OnGroupInviteAccept"},
		{"Group_DeclineInvite","OnGroupInviteAccept"}
	},
	["Summon"]={
		{"StoryPanelDialog_Show","OnStoryPanel"},
		{"ProgressClickWindowDisplay","OnWindowDisplay"}
	},
	["Ready Check"]={
		{"Group_ReadyCheck","OnReadyCheck"},
		{"Group_MemberFlagsChanged","OnGroupFlagChange"}
	},
	["Resurrection"]={
		{"ShowResurrectDialog","OnRezUpdate"},
		{"PlayerResurrected","OnRez"}
	},
	["Role Check"]={
		{"MatchingRoleCheckStarted","OnRoleCheck"},
		{"MatchingRoleCheckHidden","OnRoleChose"}
	},
	["Group Found"]={
		{"MatchingGameReady", "OnMatchGameReady"},
		{"MatchJoined", "OnMatchJoined"},
		{"MatchingLeaveQueue", "OnMatchJoined"},
		{"MatchingGameAccepted", "OnMatchJoined"}
	}
}

local function ReplaceRegex(strString)
	local strProcessed = string.gsub(strString,"[%(%)%.%%%+%-%*%?%[%^%$]", "%%%1")
	return strProcessed
end

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function WakeUp:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function WakeUp:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "WakeUp!"
	local tDependencies = {
		"MatchMaker"
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- WakeUp OnLoad
-----------------------------------------------------------------------------------------------
function WakeUp:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("WakeUp.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- WakeUp OnDocLoaded
-----------------------------------------------------------------------------------------------
function WakeUp:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("wakeup", "OnConfigure", self)
		Apollo.RegisterSlashCommand("wu", "OnConfigure", self)
		Apollo.RegisterSlashCommand("stopsound", "CancelSounds", self)
		Apollo.RegisterSlashCommand("testa", "StartRandomSound", self)
	
		--[[
		--Group Invite
		Apollo.RegisterEventHandler("Group_Invited", "OnGroupInvite", self)
		Apollo.RegisterEventHandler("Group_AcceptInvite", "OnGroupInviteAccept", self)
		
		--Summon
		Apollo.RegisterEventHandler("StoryPanelDialog_Show", "OnStoryPanel", self)
		Apollo.RegisterEventHandler("ProgressClickWindowDisplay","OnWindowDisplay", self)
		
		--Ready Check
		Apollo.RegisterEventHandler("Group_ReadyCheck", "OnReadyCheck", self)
		Apollo.RegisterEventHandler("Group_MemberFlagsChanged", "OnGroupFlagChange", self)
		
		--Rez Check
		Apollo.RegisterEventHandler("ShowResurrectDialog", "OnRezUpdate", self)
		Apollo.RegisterEventHandler("PlayerResurrected", "OnRez", self)
		
		--Role Check (Queue)
		Apollo.RegisterEventHandler("MatchingRoleCheckStarted", "OnRoleCheck", self)
		Apollo.RegisterEventHandler("MatchingRoleCheckHidden", "OnRoleChose", self)
		
		--Accept Queue
		Apollo.RegisterEventHandler("MatchingGameReady", "OnMatchGameReady", self)
		Apollo.RegisterEventHandler("MatchJoined", "OnMatchJoined", self)
		Apollo.RegisterEventHandler("MatchingLeaveQueue", "OnMatchJoined", self)
		Apollo.RegisterEventHandler("MatchingGameAccepted", "OnMatchJoined", self)
		--]]
		
		--Restore/Default Settings
		if not self.settings then
			self.settings = {}
		end
		self.settings.bRepeat = self.settings.bRepeat ~= nil and self.settings.bRepeat or true
		self.settings.nDelay = self.settings.nDelay or 3
		
		--Restore/Default Sound Settings
		if not self.settings.tEnabledSounds then
			self.settings.tEnabledSounds = TableUtil:Copy(tSounds)
		end
		
		--Restore/Default Trigger Settings
		if not self.settings.tTriggers then
			self.settings.tTriggers = {}
			for strTrigger,_ in pairs(tTriggers) do
				self.settings.tTriggers[strTrigger] = true
			end
		end
		self.settings.strTrigger = self.settings.strTrigger or "!wakeup"
		
		--Set-up Trigger Events
		for strTrigger,tTriggerEvents in pairs(tTriggers) do
			if self.settings.tTriggers[strTrigger] == true then
				for _,tTrigger in pairs(tTriggerEvents) do
					Apollo.RegisterEventHandler(tTrigger[1], tTrigger[2], self)
				end
			end
		end
		self:HookMatchMaker()
		if self.settings.bTriggerPhrase then
			Apollo.RegisterEventHandler("ChatMessage", "OnChatMessage", self)
		end
	end
end

function WakeUp:OnSave(eLevel)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
		
	return self.settings
end

function WakeUp:OnRestore(eLevel, tData)
	self.settings = tData
end

-----------------------------------------------------------------------------------------------
-- Event Handler Functions
-----------------------------------------------------------------------------------------------

--Group Invite
function WakeUp:OnGroupInvite()
	self:StartSoundCountdown(self.settings.nDelay)
end
function WakeUp:OnGroupInviteAccept()
	self:CancelSounds()
end

--Summon
function WakeUp:OnStoryPanel(eType, tLines, nDuration)
	if eType == GameLib.CodeEnumStoryPanel.Urgent then
		for _,strLine in pairs(tLines) do
			if string.find(strLine, "is trying to summon you") then
				self:StartSoundCountdown(self.settings.nDelay)
			end
		end
	end
end
function WakeUp:OnWindowDisplay(bShown)
	if not bShown then
		self:CancelSounds()
	end
end

--Ready Check
function WakeUp:OnReadyCheck(nMemberSent, strMessage)
	self:StartSoundCountdown(self.settings.nDelay)
end
function WakeUp:OnGroupFlagChange(nMember, tFlagInfo)  --TODO: NOT WORKING
	local unitSelf = GameLib.GetPlayerUnit()
	if unitSelf then
		local strMyName = unitSelf:GetName()
		local tGroupMember = GroupLib.GetGroupMember(nMember)
		if tGroupMember and tGroupMember.strCharacterName == strMyName then
			if tGroupMember.bReady then
				self:CancelSounds()
			end
		end
	end
end

--Rez Check
function WakeUp:OnRezUpdate(bPlayerIsDead, bEnableRezHere, bEnableRezHoloCrypt, bEnableRezExitInstance,
  bEnableCasterRez, bHasCasterRezRequest, nRezCost, fTimeBeforeRezable, fTimeBeforeWakeHere, fTimeBeforeForceRez)
	if bPlayerIsDead and bEnableRezHoloCrypt then
		self:StartSoundCountdown((fTimeBeforeRezable/1000)+self.settings.nDelay)
	end
end
function WakeUp:OnRez()
	self:CancelSounds()
end

--Role Check (Queue)
function WakeUp:OnRoleCheck()
	self:StartSoundCountdown(self.settings.nDelay)
end
function WakeUp:OnRoleChose()
	self:CancelSounds()
end

--Accept Queue
function WakeUp:OnMatchGameReady()
	self:StartSoundCountdown(self.settings.nDelay)
end
function WakeUp:OnMatchJoined()
	self:CancelSounds()
end
function WakeUp:OnMatchUpdate()
	local MatchMaker = Apollo.GetAddon("MatchMaker")
	if MatchMaker.wndAllyConfirm:IsVisible() then
		self:CancelSounds()
	end
end
function WakeUp:HookMatchMaker()
	local MatchMaker = Apollo.GetAddon("MatchMaker")
	MatchMaker.OnPendingGameResponded = function(self, wndHandler, wndControl, bResponse)
		if not bResponse then
			self.eSelectedTab = self.eQueuedTab
			self:OnMatchMakerOn()
		else
			self:RefreshStatus()
		end
		
		self.wndJoinGame:Show(false)
		Event_FireGenericEvent("MatchingGameAccepted")
	end
end

--Custom Trigger Phrase
function WakeUp:OnChatMessage(luaChannel, tMessageData)
	if luaChannel:GetType() == ChatSystemLib.ChatChannel_Party or luaChannel:GetType() == ChatSystemLib.ChatChannel_Instance then
		for _,tSegment in pairs(tMessageData.arMessageSegments) do
			if string.find(tSegment.strText, self.settings.strTrigger) then
				self:StartRandomSound()
				if self.settings.bRepeat and (self.wndTriggerPhrase == nil or not self.wndTriggerPhrase:IsValid())then
					self.wndTriggerPhrase = Apollo.LoadForm(self.xmlDoc, "TriggerPhraseButton", nil, self)
				end
			end
		end
	end
end

-----------------------------------------------------------------------------------------------
-- WakeUp Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

function WakeUp:StartSoundCountdown(nCountdownTime)
	if not self.tmSound and not self.tmCountdown then
		self.tmCountdown = ApolloTimer.Create(nCountdownTime, false, "StartRandomSound", self)
	end
end

function WakeUp:CancelSounds()
	self:OnStopSound()
	if self.tmCountdown then
		self.tmCountdown:Stop()
		self.tmCountdown = nil
	end
end

function WakeUp:StartRandomSound()
	math.randomseed(os.time())
	local _ = math.random(1,#self.settings.tEnabledSounds) --To eat the predictable first random.
	if not self.settings.bNavi then
		self:OnStartSound(math.random(1,#self.settings.tEnabledSounds))
	else
		self:OnStartNavi()
	end
end

function WakeUp:OnStartSound(nSound)
	self:OverrideSoundSettings()
	self.tCurrSound = self.settings.tEnabledSounds[nSound]
	self:PlayCurrSound()
	if self.settings.bRepeat then
		self.tmSound = ApolloTimer.Create(self.tCurrSound.duration, true, "PlayCurrSound", self)
	end
end

function WakeUp:PlayCurrSound()
	Sound.PlayFile("SFX\\"..self.tCurrSound.file..".wav")
end

function WakeUp:OnStopSound()
	if self.tmSound then
		self.tmSound:Stop()
		self.tmSound = nil
		self.tCurrSound = nil
	end
	self:RestoreSoundSettings()
end

function WakeUp:OverrideSoundSettings()
	self.bTabOutSound = Apollo.GetConsoleVariable("sound.playInBackground")
	self.nVolumeMaster = Apollo.GetConsoleVariable("sound.volumeMaster")
	self.nVolumeUI = Apollo.GetConsoleVariable("sound.volumeUI")
	
	Apollo.SetConsoleVariable("sound.playInBackground", true)
	if self.nVolumeMaster < 0.10 then
		Apollo.SetConsoleVariable("sound.volumeMaster", 1) --TODO: May destroy eardrums.
	end
	Apollo.SetConsoleVariable("sound.volumeUI", 1)
end

function WakeUp:RestoreSoundSettings()
	Apollo.SetConsoleVariable("sound.playInBackground", self.bTabOutSound)
	Apollo.SetConsoleVariable("sound.volumeMaster", self.nVolumeMaster)
	Apollo.SetConsoleVariable("sound.volumeUI", self.nVolumeUI)
end


--Navi Stuff
function WakeUp:OnStartNavi()
	self:OverrideSoundSettings()
	self:PlayNavi()
	if self.settings.bRepeat then
		self.tmSound = ApolloTimer.Create(1, self.settings.bRepeat, "PlayNavi", self)
	end
end

function WakeUp:PlayNavi()
	local _ = math.random(1,#tNavi) --To eat the predictable first random.
	Sound.PlayFile("SFX\\"..tNavi[math.random(1,#tNavi)]..".wav")
end


-----------------------------------------------------------------------------------------------
-- WakeUpForm Functions
-----------------------------------------------------------------------------------------------

function WakeUp:OnConfigure()
	if not self.wndMain or not self.wndMain:IsValid() then
		self.wndMain = Apollo.LoadForm(self.xmlDoc, "WakeUpForm", nil, self)
	end
	--Sounds
	local wndSounds = self.wndMain:FindChild("Sounds")
	wndSounds:DestroyChildren()
	for _,tSound in pairs(tSounds) do
		local wndSound = Apollo.LoadForm(self.xmlDoc, "SoundItem", wndSounds, self)
		wndSound:SetData(tSound)
		wndSound:FindChild("SoundName"):SetText(tSound.name)
		for idx,tEnabledSound in pairs(self.settings.tEnabledSounds) do
			if tEnabledSound.file == tSound.file then
				wndSound:FindChild("Check"):Show(true)
				wndSound:SetCheck(true)
				break
			end
		end
	end
	wndSounds:ArrangeChildrenVert()
	--Delay
	self.wndMain:FindChild("Delay"):SetText(self.settings.nDelay)
	--Repeat
	self.wndMain:FindChild("Repeat"):SetCheck(self.settings.bRepeat)
	--Trigger Phrase
	self.wndMain:FindChild("TriggerPhraseBtn"):SetCheck(self.settings.bTriggerPhrase)
	self.wndMain:FindChild("TriggerPhrase"):SetText(self.settings.strTrigger)
	--Navi
	self.wndMain:FindChild("Blocker"):Show(self.settings.bNavi)
	self.wndMain:FindChild("NaviBtn"):SetCheck(self.settings.bNavi)
	--Triggers
	local strPlayerName = GameLib.GetPlayerUnit():GetName()
	if strPlayerName == "Pohka Hauntess" or strPlayerName == "Pokka Fera" or strPlayerName == "Pohka Dots" then self.wndMain:FindChild("Repeat"):Show(false) return end --LEL
	
	local wndTriggers = self.wndMain:FindChild("Triggers")
	wndTriggers:DestroyChildren()
	for strTrigger,_ in pairs(tTriggers) do
		local wndTrigger = Apollo.LoadForm(self.xmlDoc, "TriggerItem", wndTriggers, self)
		wndTrigger:SetText(strTrigger)
		wndTrigger:SetCheck(self.settings.tTriggers[strTrigger])
	end
	wndTriggers:ArrangeChildrenVert()
end

function WakeUp:OnClose( wndHandler, wndControl )
	self.wndMain:Destroy()
end

---- Options -------------------------------------------------------------

--Delay
function WakeUp:OnDelayChanging( wndHandler, wndControl, strText )
	local strReplace = string.gsub(strText,"[^0-9]","")
	if strText ~= strReplace then
		wndHandler:SetText(strReplace)
		wndHandler:SetSel(#strReplace,#strReplace)
		Print("[WakeUp!] Delay is a number in seconds, not letters!")
	end
end
function WakeUp:OnDelayChange( wndHandler, wndControl, strText )
	if tonumber(strText) then
		self.settings.nDelay = tonumber(strText)
	else
		Print("[WakeUp!] Invalid delay time. Try using only numbers.")
		wndHandler:SetText(self.settings.nDelay)
	end
end
function WakeUp:OnDelayEscape( wndHandler, wndControl )
	wndHandler:SetText(self.settings.nDelay)
	Print("[WakeUp!] Delay reset to previous. You must hit ENTER (Return) to save delay time.")
end

--Repeat
function WakeUp:OnToggleRepeat( wndHandler, wndControl, eMouseButton )
	self.settings.bRepeat = wndHandler:IsChecked()
end

--Trigger Phrase
function WakeUp:OnToggleTriggerPhrase( wndHandler, wndControl, eMouseButton )
	local bChecked = wndHandler:IsChecked()
	self.settings.bTriggerPhrase = bChecked
	
	if bChecked then
		Apollo.RegisterEventHandler("ChatMessage", "OnChatMessage", self)
	else
		Apollo.RemoveEventHandler("ChatMessage", self)
	end
end
function WakeUp:OnTriggerChange( wndHandler, wndControl, strText )
	if strText ~= "" and strText ~= nil then
		self.settings.strTrigger = strText
	else
		Print("[WakeUp!] Invalid trigger phrase. Try again.")
		wndHandler:SetText(self.settings.nDelay)
	end
end
function WakeUp:OnTriggerEscape( wndHandler, wndControl )
	wndHandler:SetText(self.settings.strTrigger)
	--wndHandler:SetSel(#self.settings.strTrigger, #self.settings.strTrigger)
	Print("[WakeUp!] Trigger phrase reset to previous. You must hit ENTER (Return) to save trigger phrase.")
end


--Navi
function WakeUp:OnToggleNavi( wndHandler, wndControl, eMouseButton )
	self.settings.bNavi = wndHandler:IsChecked()
	self.wndMain:FindChild("Blocker"):Show(self.settings.bNavi)
end

---------------------------------------------------------------------------------------------------
-- SoundItem Functions
---------------------------------------------------------------------------------------------------

function WakeUp:OnSoundToggle( wndHandler, wndControl, eMouseButton )
	local bEnabled = wndHandler:IsChecked()
	local tSound = wndHandler:GetData()

	if bEnabled then
		table.insert(self.settings.tEnabledSounds, tSound)
	else
		if #self.settings.tEnabledSounds > 1 then
			for idx,tEnabledSound in pairs(self.settings.tEnabledSounds) do
				if tSound.file == tEnabledSound.file then
					table.remove(self.settings.tEnabledSounds, idx)
					break
				end
			end
		else
			Print("[WakeUp!] You must have at least 1 sound enabled!")
			wndHandler:SetCheck(true)
			return
		end
	end
	wndHandler:FindChild("Check"):Show(bEnabled)
end

---------------------------------------------------------------------------------------------------
-- TriggerItem Functions
---------------------------------------------------------------------------------------------------

function WakeUp:OnToggleTrigger( wndHandler, wndControl, eMouseButton )
	local strTrigger = wndHandler:GetText()
	local bChecked = wndHandler:IsChecked()
	self.settings.tTriggers[strTrigger] = bChecked
	
	if bChecked then
		for _,tTriggerEvents in pairs(tTriggers[strTrigger]) do
			Apollo.RegisterEventHandler(tTriggerEvents[1], tTriggerEvents[2], self)
		end		
	else
		for _,tTriggerEvents in pairs(tTriggers[strTrigger]) do
			Apollo.RemoveEventHandler(tTriggerEvents[1], self)
		end		
	end
end

---------------------------------------------------------------------------------------------------
-- TriggerPhraseButton Functions
---------------------------------------------------------------------------------------------------

function WakeUp:OnTriggerPhraseDone( wndHandler, wndControl, eMouseButton )
	self.wndTriggerPhrase:Destroy()
	self.wndTriggerPhrase = nil
	self:CancelSounds()
end

-----------------------------------------------------------------------------------------------
-- WakeUp Instance
-----------------------------------------------------------------------------------------------
local WakeUpInst = WakeUp:new()
WakeUpInst:Init()
